#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Define your custom views here or overwrite the default views. Default
# CRUD operations are generated by the ringo frameworkd.
import logging
from sets import Set
from pyramid.view import view_config
from formbar.form import Validator
from ringo.lib.helpers import get_action_routename
from ringo_eccourse.model import Eccourse
import ringo_eccourse.lib.plz as eccourse_plz
from ringo.views.base import create, update

log = logging.getLogger(name=__name__)


def plz2locations(plz):
    eccourse = eccourse_plz.get(plz)
    if eccourse:
        return Set([x["location"] for x in eccourse])
    return []


# plz validator code.
def check_plz_validator(field, data, context):
    plz_data = data.get(field)
    return plz_data.isdigit() and eccourse_plz.is_valid(plz_data)


def plz_validator_factory(request):
    plz_data = request.params.get('plz')
    _ = request.translate
    return Validator('plz', (_(u'"%s" is not a valid PLZ.') % plz_data),
                     check_plz_validator, request)


def location_plz_uniqueness_validator(field, data, context):
    '''
    Helper validator that checks that a PLZ has a unique location mappin
    '''
    plz = data.get('plz')
    if not plz:
        return True
    else:
        eccourse = eccourse_plz.get(plz)
        valid_locations_for_plz = plz2locations(plz)
        if not eccourse:
            # Mark location field as invalid and allow user to edit the
            # value manually.
            data["_location_valid"] = 0
            data["location"] = ""
            data["nuts3"] = ""
            data["region_category"] = ""
            data["federal_state"] = ""
            return False

        # Only one entry, take everything
        if len(eccourse) == 1:
            data['location'] = eccourse[0].get("location")
            return True
        elif data["location"] and data["location"] in valid_locations_for_plz:
            return True
        else:
            # Mark location field as invalid and allow user to edit the
            # value manually.
            data["_location_valid"] = 0

            data["location"] = ""
            data["nuts3"] = ""
            data["region_category"] = ""
            data["federal_state"] = ""
            return False


def nuts3_validator(field, data, context):
    """This validator does not validate but doest the mapping from the
    plz and location to nuts3. The nuts3 code is then used to fill the
    region_category and the federal_state. Because no validation is done
    the method will allways return True. Validation is already done
    before."""

    def location2nuts3(eccourse, location):
        for entry in eccourse:
            if entry["location"] == location:
                return entry["nuts3"]
        return None

    def nuts2rc(eccourse, nutscode):
        for entry in eccourse:
            if entry["nuts3"] == nutscode:
                return entry["region_category"]
        return None

    def nuts2fc(eccourse, nutscode):
        for entry in eccourse:
            if entry["nuts3"] == nutscode:
                return entry["federal_state"]
        return None

    def nuts2location(eccourse, nutscode):
        for entry in eccourse:
            if entry["nuts3"] == nutscode:
                return entry["location"]
        return None

    plz = data.get('plz')
    location = data.get('location')

    if (plz and location):
        eccourse = eccourse_plz.get(plz)
        nuts3 = location2nuts3(eccourse, location)
        if eccourse and nuts3:
            data['nuts3'] = nuts3
            data['region_category'] = nuts2rc(eccourse, nuts3)
            data['federal_state'] = nuts2fc(eccourse, nuts3)
    return True


def location_plz_uniqueness_validator_factory(request):
    plz_data = request.params.get('plz')
    locations = plz2locations(plz_data)
    _ = request.translate
    if locations:
        locations = ", ".join([u'„{}“'.format(x) for x in locations])
    else:
        locations = _('No locations found.')
    return Validator('location', (_(u'We can not find a unique location code mapping for the given PLZ %s. '
                                    u'Please choose one of the following locations: %s') % (plz_data, locations)),
                     location_plz_uniqueness_validator, request, triggers="error")


# views
@view_config(route_name=get_action_routename(Eccourse, 'create'),
             renderer='/default/create.mako',
             permission='create')
def create_(request):

    return create(request, validators=[plz_validator_factory(request),
                                       location_plz_uniqueness_validator_factory(request),
                                       Validator('nuts3', "", nuts3_validator, request)])


@view_config(route_name=get_action_routename(Eccourse, 'update'),
             renderer='/default/update.mako',
             permission='update')
def update_(request):
    return update(request, validators=[plz_validator_factory(request),
                                       location_plz_uniqueness_validator_factory(request),
                                       Validator('nuts3', "", nuts3_validator, request)])
